@@ -24,6 +24,7 @@ import thread
 import time
 import gettext
 import sys
+import tempfile
 import pwd
 
 from gettext import gettext as _
@@ -104,6 +105,7 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGladeApp):
         combo.add_attribute(cell, 'text', COMBO_LANGUAGE)
         combo.set_model(model)
         self.combo_dirty = False
+        self.imSwitch = ImSwitch()
 
         # apply button
         self.button_apply.set_sensitive(False)
@@ -113,24 +115,58 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGladeApp):
 
         # show it
         self.window_main.show()
-        self.window_main.set_sensitive(False)
-        watch = gtk.gdk.Cursor(gtk.gdk.WATCH)
-        self.window_main.window.set_cursor(watch)
-        while gtk.events_pending():
-            gtk.main_iteration()
-        
+        self.setSensitive(False)
+       
         self.updateLanguageView()
         self.updateSystemDefaultCombo()
-        
+
+        # check if the package list is up-to-date
+        if not self.verifyPackageLists():
+            d = gtk.MessageDialog(parent=self.window_main,
+                                  flags=gtk.DIALOG_MODAL,
+                                  type=gtk.MESSAGE_INFO,
+                                  buttons=gtk.BUTTONS_CANCEL)
+            d.set_markup("<big><b>%s</b></big>\n\n%s" % (
+                _("No language information available"),
+                _("The system does not have information about the "
+                  "available languages yet. Do you want to perform "
+                  "a network update to get them now? ")))
+            d.set_title=("")
+            d.add_button(_("_Update"), gtk.RESPONSE_YES)
+            res = d.run()
+            d.destroy()
+            if res == gtk.RESPONSE_YES:
+                self.setSensitive(False)
+                self.update()
+                self.updateLanguageView()
+                self.setSensitive(True)
+
         # see if something is missing
         if options.verify_installed:
             self.verifyInstalledLangPacks()
-        self.window_main.set_sensitive(True)
-        self.window_main.window.set_cursor(None)
 
-        if (not os.path.exists("/etc/alternatives/xinput-all_ALL") or
-            not os.path.exists("/usr/bin/im-switch")):
+        if not self.imSwitch.available():
             self.checkbutton_enable_input_methods.set_sensitive(False)
+        self.setSensitive(True)
+
+    def setSensitive(self, value):
+        if value:
+            self.window_main.set_sensitive(True)
+            self.window_main.window.set_cursor(None)
+        else:
+            self.window_main.set_sensitive(False)
+            self.window_main.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
+        while gtk.events_pending():
+            gtk.main_iteration()
+
+    def runAsRoot(self, userCmd):
+        " run the given command as root using gksu "
+        cmd = ["/usr/bin/gksu", 
+               "--desktop", 
+               "/usr/share/applications/language-selector.desktop", 
+               "--"]
+        ret = subprocess.call(cmd+userCmd)
+        return (ret == 0)
 
     def setupTreeView(self):
         """ do all the treeview setup here """
@@ -138,30 +174,32 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGladeApp):
             langInfo = model.get_value(iter, LIST_LANG_INFO)
 
             # check for active and inconsitent 
-            inconsistent = xor(langInfo.langPackInstalled, langInfo.langSupportInstalled) 
+            inconsistent = langInfo.inconsistent
             #if inconsistent:
             #    print "%s is inconsistent" % langInfo.language
 
             # do we want to install or remove it?
-            toInstall = (langInfo.installLangPack and \
-                         langInfo.installLangSupport)
-            toRemove = (not langInfo.installLangPack and \
-                        not langInfo.installLangSupport)
+            #toInstall = (langInfo.languagePkgList["languagePack"].doChange and \
+            #            langInfo.languagePkgList["languageSupport"].doChange)
+            #toRemove = (not langInfo.languagePkgList["languagePack"].doChange and \
+            #            not langInfo.languagePkgList["languagePack"].doChange)
             # if it is going to be installed or removed, it can't
             # be inconsitent
-            if inconsistent and (toInstall or toRemove):
-                inconsistent = False
+            #if inconsistent and (toInstall or toRemove):
+            #    inconsistent = False
                 
-            cell.set_property("active", toInstall)
+            #cell.set_property("active", toInstall)
+            cell.set_property("active", langInfo.fullInstalled)
             cell.set_property("inconsistent", inconsistent)
             
         def lang_view_func(cell_layout, renderer, model, iter):
             langInfo = model.get_value(iter, LIST_LANG_INFO)
-            inconsistent = xor(langInfo.langPackInstalled, langInfo.langSupportInstalled)
-            current = langInfo.langPackInstalled or langInfo.langSupportInstalled
-            toInstall = langInfo.installLangPack and langInfo.installLangSupport
-            toRemove =  not langInfo.installLangPack and not langInfo.installLangSupport
-            if (not inconsistent and current != toInstall) or (inconsistent and (toInstall or toRemove)):
+            inconsistent = langInfo.inconsistent
+            #current = langInfo.languagePkgList["languagePack"].installed or langInfo.languagePkgList["languageSupport"].installed
+            #toInstall = langInfo.languagePkgList["languagePack"].doChange and langInfo.languagePkgList["languageSupport"].doChange
+            #toRemove =  not langInfo.languagePkgList["languagePack"].doChange and not langInfo.languagePkgList["languageSupport"].doChange
+            #if (not inconsistent and current != toInstall) or (inconsistent and (toInstall or toRemove)):
+            if (langInfo.changes) :
                 markup = "<b>%s</b>" % langInfo.language
             else:
                 markup = "%s" % langInfo.language
@@ -184,34 +222,124 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGladeApp):
         self._langlist = gtk.ListStore(str, gobject.TYPE_PYOBJECT)
         self.treeview_languages.set_model(self._langlist)
 
+    def _get_langinfo_on_cursor(self):
+        (path, column) = self.treeview_languages.get_cursor ()
+        if not path:
+            return None
+        iter = self._langlist.get_iter(path)
+        langInfo = self._langlist.get_value(iter, LIST_LANG_INFO)
+        return langInfo
 
+    # details checkboxes
+    def on_checkbutton_fonts_clicked(self, button):
+        if self.block_toggle: return
+        langInfo = self._get_langinfo_on_cursor()
+        langInfo.languagePkgList["languageSupportFonts"].doChange = not langInfo.languagePkgList["languageSupportFonts"].doChange
+        self.check_apply_button()
+        self.treeview_languages.queue_draw()
+        #self.debug_pkg_status()
+    def on_checkbutton_input_methods_clicked(self, button):
+        if self.block_toggle: return
+        langInfo = self._get_langinfo_on_cursor()
+        langInfo.languagePkgList["languageSupportInputMethods"].doChange = not langInfo.languagePkgList["languageSupportInputMethods"].doChange
+        self.check_apply_button()
+        self.treeview_languages.queue_draw()
+        #self.debug_pkg_status()
+    def on_checkbutton_writing_aids_clicked(self, button):
+        if self.block_toggle: return
+        langInfo = self._get_langinfo_on_cursor()
+        langInfo.languagePkgList["languageSupportWritingAids"].doChange = not langInfo.languagePkgList["languageSupportWritingAids"].doChange
+        self.check_apply_button()
+        self.treeview_languages.queue_draw()
+        #self.debug_pkg_status()
+    def on_checkbutton_basic_translations_clicked(self, button):
+        if self.block_toggle: return
+        langInfo = self._get_langinfo_on_cursor()
+        langInfo.languagePkgList["languagePack"].doChange = not langInfo.languagePkgList["languagePack"].doChange
+        self.check_apply_button()
+        self.treeview_languages.queue_draw()
+        #self.debug_pkg_status()
+    def on_checkbutton_extra_translations_clicked(self, button):
+        if self.block_toggle: return
+        langInfo = self._get_langinfo_on_cursor()
+        langInfo.languagePkgList["languageSupportTranslations"].doChange = not langInfo.languagePkgList["languageSupportTranslations"].doChange
+        self.check_apply_button()
+        self.treeview_languages.queue_draw()
+        #self.debug_pkg_status()
+    def on_checkbutton_extra_clicked(self, button):
+        if self.block_toggle: return
+        langInfo = self._get_langinfo_on_cursor()
+        langInfo.languagePkgList["languageSupportExtra"].doChange = not langInfo.languagePkgList["languageSupportExtra"].doChange
+        self.check_apply_button()
+        self.treeview_languages.queue_draw()
+        #self.debug_pkg_status()
+
+    # the global toggle
     def on_toggled(self, renderer, path_string):
         """ called when on install toggle """
         iter = self._langlist.get_iter_from_string(path_string)
         langInfo = self._langlist.get_value(iter, LIST_LANG_INFO)
 
         # special handling for inconsistent state
-        inconsistent = xor(langInfo.langPackInstalled, langInfo.langSupportInstalled)
+        #inconsistent = False  #xor(langInfo.languagePkgList["languagePack"].installed, langInfo.languagePkgList["languageSupport"].installed)
         # check if we already set remove 
-        toInstall = langInfo.installLangPack and langInfo.installLangSupport
-        toRemove = not langInfo.installLangPack and not langInfo.installLangSupport
+        #toInstall = langInfo.languagePkgList["languagePack"].doChange and langInfo.languagePkgList["languageSupport"].doChange
+        #toRemove = not langInfo.languagePkgList["languagePack"].doChange and not langInfo.languagePkgList["languageSupport"].doChange
         # if it is inconistent and wasn't touched yet, set it for install
         # from then on it will just work (tm)
-        if inconsistent and not (toInstall or toRemove):
-            langInfo.installLangPack = True
-            langInfo.installLangSupport = True
-        else:
-            langInfo.installLangPack = not langInfo.installLangPack
-            langInfo.installLangSupport = not langInfo.installLangSupport
+        if langInfo.inconsistent :
+            for pkg in langInfo.languagePkgList.values() :
+                if (pkg.available and not pkg.installed) : 
+                    pkg.doChange = True
+        elif langInfo.fullInstalled :
+            for pkg in langInfo.languagePkgList.values() :
+                if (pkg.available) :
+                    if (not pkg.installed and pkg.doChange) :
+                        pkg.doChange = False
+                    elif (pkg.installed and not pkg.doChange) :
+                        pkg.doChange = True
+        else :
+            for pkg in langInfo.languagePkgList.values() :
+                if (pkg.available) :
+                    if (pkg.installed and pkg.doChange) :
+                        pkg.doChange = False
+                    elif (not pkg.installed and not pkg.doChange) :
+                        pkg.doChange = True
+
         self.check_apply_button()
+        self.treeview_languages.queue_draw()
+        #self.debug_pkg_status()
+
+    def on_treeview_languages_cursor_changed(self, treeview):
+        #print "on_treeview_languages_cursor_changed()"
+        langInfo = self._get_langinfo_on_cursor()
+        for (button, attr) in ( 
+              ("checkbutton_basic_translations", langInfo.languagePkgList["languagePack"]),
+              ("checkbutton_extra_translations", langInfo.languagePkgList["languageSupportTranslations"]),
+              ("checkbutton_writing_aids", langInfo.languagePkgList["languageSupportWritingAids"]),
+              ("checkbutton_input_methods", langInfo.languagePkgList["languageSupportInputMethods"]),
+              ("checkbutton_extra", langInfo.languagePkgList["languageSupportExtra"]),
+              ("checkbutton_fonts", langInfo.languagePkgList["languageSupportFonts"])  ):
+            self.block_toggle = True
+            if ((attr.installed and not attr.doChange) or (not attr.installed and attr.doChange)) :
+                getattr(self, button).set_active(True)
+            else :
+                getattr(self, button).set_active(False)
+            getattr(self, button).set_sensitive(attr.available)
+            self.block_toggle = False
+            #self.debug_pkg_status()
+
+    def debug_pkg_status(self):
+        langInfo = self._get_langinfo_on_cursor()
+        for pkg in langInfo.languagePkgList.items() :
+            print ("%s, available: %s, installed: %s, doChange: %s" % (pkg[0], pkg[1].available, pkg[1].installed, pkg[1].doChange))
+        print ("inconsistent? : %s" % langInfo.inconsistent)
 
     def check_apply_button(self):
         changed = False
         for (lang, langInfo) in self._langlist:
-            if (langInfo.langPackInstalled != langInfo.installLangPack) or \
-               (langInfo.langSupportInstalled != langInfo.installLangSupport):
+            if langInfo.changes:
                 changed = True
-        
         if self.combo_dirty or changed:
             self.button_apply.set_sensitive(True)
         else:
@@ -226,83 +354,63 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGladeApp):
         """ check if the selected langauge has input method support
             and set checkbutton_enable_input_methods accordingly
         """
-        if (os.path.exists("/etc/alternatives/xinput-all_ALL") and
-            os.path.exists("/usr/bin/im-switch")):
-            combo = self.combobox_default_lang
-            model = combo.get_model()
-            if combo.get_active() < 0:
-                return
-            (lang, code) = model[combo.get_active()]
-            # if we have a default in im-switch for this language, we
-            # can't change it from here (unless the im-switch swamp in drained)
-            default = os.path.exists("/etc/alternatives/xinput-%s" % code)
-            self.checkbutton_enable_input_methods.set_active(default)
-            self.checkbutton_enable_input_methods.set_sensitive(not default)
-            if default:
-                return
-            # now check if we have overwritten this - we do this by checking
-            # the setting for all_ALL (im-switch goodness again :/)
-            target = os.path.basename(os.readlink("/etc/alternatives/xinput-all_ALL"))
-            active = (target != "default" and target != "none")
-            self.checkbutton_enable_input_methods.set_active(active)
+        if not self.imSwitch.available():
+            return
+        # get current selected default language
+        combo = self.combobox_default_lang
+        model = combo.get_model()
+        if combo.get_active() < 0:
+            return
+        (lang, code) = model[combo.get_active()]
+        # check if that has a im-switch config
+        active = self.imSwitch.enabledForLocale(code)
+        self._blockSignals = True
+        self.checkbutton_enable_input_methods.set_active(active)
+        self._blockSignals = False
 
-    def __input_method_config_changed(self):
-        """ check if we changed the input method config here         
-        """
-        if (os.path.exists("/etc/alternatives/xinput-all_ALL") and
-            os.path.exists("/usr/bin/im-switch")):
-            combo = self.combobox_default_lang
-            model = combo.get_model()
-            (lang, code) = model[combo.get_active()]
-            # if we have a default in im-switch for this language, we
-            # can't change it from here (unless the im-switch swamp in drained)
-            default = os.path.exists("/etc/alternatives/xinput-%s" % code)
-            if default:
-                return
-            # now check if we have overwritten this - we do this by checking
-            # the setting for all_ALL (im-switch goodness again :/)
-            target = os.path.basename(os.readlink("/etc/alternatives/xinput-all_ALL"))
-            current = (target != "default" and target != "none")
-            new = self.checkbutton_enable_input_methods.get_active()
-            return (new != current)
-
-    def updateInputMethods(self):
-        """ write new input method defaults - currently we only support
-            "im-switch -a" to reset and
-            "im-switch -s scim" to set to scim (that is the default im
-                              for all CJK languages currently)
+    def writeInputMethodConfig(self):
+        """ 
+        write new input method defaults - currently we only support all_ALL
         """
         combo = self.combobox_default_lang
         model = combo.get_model()
         if combo.get_active() < 0:
             return
         (lang, code) = model[combo.get_active()]
-        # if something has changed - act!
-        if self.__input_method_config_changed():
-            if self.checkbutton_enable_input_methods.get_active():
-                os.system("im-switch -z %s -s scim" % code)
+        # check if we need to do something
+        new_value = self.checkbutton_enable_input_methods.get_active()
+        if self.imSwitch.enabledForLocale(code) != new_value:
+            if new_value:
+                self.imSwitch.enable(code)
             else:
-                os.system("im-switch -z %s -a" % code)
+                self.imSwitch.disable(code)
+            #self.showRebootRequired()
+            self.checkReloginNotification()
 
     def on_checkbutton_enable_input_methods_toggled(self, widget):
-        if self.__input_method_config_changed():
-            self.combo_dirty = True
-            self.check_apply_button()
+        if self._blockSignals:
+            return
+        #print "on_checkbutton_enable_input_methods_toggled()"
+        active = self.checkbutton_enable_input_methods.get_active()
+        self.combo_dirty = True
+        self.check_apply_button()
 
     def build_commit_lists(self):
+        # FIXME: add support to install only subsets of the 
+        #        language - use langInfo.languagePkgList["languageSupport"]{Fonts,WritingAids,..}.{install,doChange} for this
         for (lang, langInfo) in self._langlist:
-            inconsistent = xor(langInfo.langPackInstalled, langInfo.langSupportInstalled)
-            
-            wasInstalled = langInfo.langPackInstalled and langInfo.langSupportInstalled
-            wasNotInstalled = not langInfo.langPackInstalled and not langInfo.langSupportInstalled
+            #inconsistent = xor(langInfo.languagePkgList["languagePack"].installed, langInfo.languagePkgList["languageSupport"].installed)
 
-            toInstall = langInfo.installLangPack and langInfo.installLangSupport
-            toRemove = not langInfo.installLangPack and not langInfo.installLangSupport
-            
-            if (inconsistent and toInstall) or (toInstall and wasNotInstalled):
-                self._cache.tryInstallLanguage(langInfo.languageCode)
-            if (inconsistent and toRemove) or (toRemove and wasInstalled):
-                self._cache.tryRemoveLanguage(langInfo.languageCode)
+            #wasInstalled = langInfo.languagePkgList["languagePack"].installed and langInfo.languagePkgList["languageSupport"].installed
+            #wasNotInstalled = not langInfo.languagePkgList["languagePack"].installed and not langInfo.languagePkgList["languageSupport"].installed
+
+            #toInstall = langInfo.languagePkgList["languagePack"].doChange and langInfo.languagePkgList["languageSupport"].doChange
+            #toRemove = not langInfo.languagePkgList["languagePack"].doChange and not langInfo.languagePkgList["languageSupport"].doChange
+            #if (inconsistent and toInstall) or (toInstall and wasNotInstalled):
+            #    self._cache.tryInstallLanguage(langInfo.languageCode)
+            #if (inconsistent and toRemove) or (toRemove and wasInstalled):
+            #    self._cache.tryRemoveLanguage(langInfo.languageCode)
+            self._cache.tryChangeDetails(langInfo)
         (to_inst, to_rm) = self._cache.getChangesList()
         #print "inst: %s" % to_inst
         #print "rm: %s" % to_rm
@@ -347,8 +455,14 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGladeApp):
             res = False
         return res
 
-    def _commit(self):
-        """ commit helper, builds the commit lists, verifies it """
+    def commitAllChanges(self):
+        """ 
+        commit helper, builds the commit lists, verifies it
+        
+        returns the number of install/removed packages
+        """
+        self.setSensitive(False)
+        # install the new packages (if any)
         (inst_list, rm_list) = self.build_commit_lists()
         if not self.verify_commit_lists(inst_list, rm_list):
             d = gtk.MessageDialog(parent=self.window_main,
@@ -363,69 +477,119 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGladeApp):
             d.set_title("")
             d.run()
             d.destroy()
-            return False
+            self.setSensitive(True)
+            return 0
         #print "inst_list: %s " % inst_list
         #print "rm_list: %s " % rm_list
         self.commit(inst_list, rm_list)
-        self.writeSystemDefaultLang()
-        # queue a restart of gdm (if it is runing) to make the new
-        # locales usable
-        gdmscript = "/etc/init.d/gdm"
-        if os.path.exists("/var/run/gdm.pid") and os.path.exists(gdmscript):
-            subprocess.call(["invoke-rc.d","gdm","reload"])
-        return True
+
+        # write input method config
+        self.writeInputMethodConfig()
+
+        # write the system default language
+        if self.writeSystemDefaultLang():
+            # queue a restart of gdm (if it is runing) to make the new
+            # locales usable
+            gdmscript = "/etc/init.d/gdm"
+            if os.path.exists("/var/run/gdm.pid") and os.path.exists(gdmscript):
+                self.runAsRoot(["invoke-rc.d","gdm","reload"])
+        self.setSensitive(True)
+        return len(inst_list)+len(rm_list)
 
     def on_button_ok_clicked(self, widget):
-        self._commit()
+        self.commitAllChanges()
         gtk.main_quit()
 
     def on_button_apply_clicked(self, widget):
-        self._commit()
-        self.updateLanguageView()
-        self.updateInputMethods()
+        if self.commitAllChanges() > 0:
+            self.updateLanguageView()
         self.updateSystemDefaultCombo()
 
-    def run_synaptic(self, lock, inst, rm, id):
-        cmd = ["/usr/sbin/synaptic", "--hide-main-window",
-               "--non-interactive", "--set-selections",
+    def _run_synaptic(self, lock, inst, rm, id):
+        # FIXME: use self.runAsRoot() here
+        cmd = ["gksu", 
+               "--desktop", "/usr/share/applications/language-selector.desktop", 
+               "--",
+               "/usr/sbin/synaptic", "--hide-main-window",
+               "--non-interactive", 
                "--parent-window-id", "%s" % (id),
                "--finish-str", _("The list of available languages on the "
                                  "system has been updated.")
                ]
-        proc = subprocess.Popen(cmd, stdin=subprocess.PIPE)
-        f = proc.stdin
+        f = tempfile.NamedTemporaryFile()        
+        cmd.append("--set-selections-file")
+        cmd.append(f.name)
         for s in inst:
             f.write("%s\tinstall\n" % s)
         for s in rm:
             f.write("%s\tdeinstall\n" % s)
-        f.close()
-        self.install_result = proc.wait()
+        f.flush()
+        subprocess.call(cmd)
         lock.release()
 
-    def commit(self, inst, rm):
-        # unlock here to make sure that lock/unlock are always run
-        # pair-wise (and don't explode on errors)
-        try:
-            apt_pkg.PkgSystemUnLock()
-        except SystemError:
-            print "WARNING: trying to unlock a not-locked PkgSystem"
-            pass
-        if len(inst) == 0 and len(rm) == 0:
-            return
-        self.window_main.set_sensitive(False)
+    def _run_synaptic_update(self, lock, id):
+        cmd = ["gksu", 
+               "--desktop", "/usr/share/applications/language-selector.desktop", 
+               "--",
+               "/usr/sbin/synaptic", "--hide-main-window",
+               "--parent-window-id", "%s" % (id),
+               "--non-interactive", "--update-at-startup"
+               ]
+        subprocess.call(cmd)
+        lock.release()
+
+    def update(self):
+        " update the package lists via synaptic "
         lock = thread.allocate_lock()
         lock.acquire()
-        t = thread.start_new_thread(self.run_synaptic,(lock,inst,rm, self.window_main.window.xid))
+        t = thread.start_new_thread(self._run_synaptic_update,(lock, self.window_main.window.xid))
         while lock.locked():
             while gtk.events_pending():
                 gtk.main_iteration()
             time.sleep(0.05)
-        # set reboot required marker if needed
-        self.rebootNotification()
-        self.window_main.set_sensitive(True)
-        while gtk.events_pending():
-            gtk.main_iteration()
-                    
+
+    def commit(self, inst, rm):
+        # unlock here to make sure that lock/unlock are always run
+        # pair-wise (and don't explode on errors)
+        if len(inst) == 0 and len(rm) == 0:
+            return
+        lock = thread.allocate_lock()
+        lock.acquire()
+        t = thread.start_new_thread(self._run_synaptic,(lock,inst,rm, self.window_main.window.xid))
+        while lock.locked():
+            while gtk.events_pending():
+                gtk.main_iteration()
+            time.sleep(0.05)
+
+    def checkReloginNotification(self):
+        #" display a reboot required notification "
+        #rb = "/usr/share/update-notifier/notify-reboot-required"
+        #if (hasattr(self, "install_result") and 
+        #    self.install_result == 0 and
+        #    os.path.exists(rb)):
+        #    self.runAsRoot([rb])
+        #    s = "/var/lib/update-notifier/dpkg-run-stamp"
+        #    if os.path.exists(s):
+        #        self.runAsRoot(["touch",s])
+        #return True
+        language_support_dir = '/usr/share/language-support'
+        update_notifier_dir = '/var/lib/update-notifier/user.d'
+        note = 'restart_session_required.note'
+        notepath = os.path.join(language_support_dir, note)
+        if os.path.exists(notepath):
+            if not os.path.exists(update_notifier_dir):
+                runAsRoot(["mkdir",update_notifier_dir])
+            s = os.path.join(update_notifier_dir, note)
+            if os.path.exists(s):
+                self.runAsRoot(["touch",s])
+            else :
+                rb = "cp %s %s" % (notepath, s)
+                self.runAsRoot([rb])
+            s = "/var/lib/update-notifier/dpkg-run-stamp"
+            if os.path.exists(s):
+                self.runAsRoot(["touch",s])
+        return True
+
     def on_button_cancel_clicked(self, widget):
         #print "button_cancel"
         gtk.main_quit()
@@ -476,7 +640,7 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGladeApp):
             buf = textview.get_buffer()
             pkgs = ""
             for pkg in missing:
-                pkgs += "%s (%s)\n" % (pkg, apt.SizeToStr(self._cache[pkg].packageSize))
+                pkgs += "%s\n" % pkg
             buf.set_text(pkgs)
             buf.place_cursor(buf.get_start_iter())
             expander.add(scroll)
@@ -486,30 +650,13 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGladeApp):
             res = d.run()
             d.destroy()
             if res == gtk.RESPONSE_YES:
+                self.setSensitive(False)
                 self.commit(missing, [])
                 self.updateLanguageView()
+                self.setSensitive(True)
 
     def updateLanguageView(self):
         #print "updateLanguageView()"
-
-        # get the lock
-        try:
-            apt_pkg.PkgSystemLock()
-        except SystemError:
-            d = gtk.MessageDialog(parent=self.window_main,
-                                  flags=gtk.DIALOG_MODAL,
-                                  type=gtk.MESSAGE_ERROR,
-                                  buttons=gtk.BUTTONS_CLOSE)
-            d.set_markup("<big><b>%s</b></big>\n\n%s" % (
-                _("Only one software management tool is allowed to"
-                  " run at the same time"),
-                _("Please close the other application e.g. \"Update "
-                  "Manager\", \"aptitude\" or \"Synaptic\" at first.")))
-            d.set_title("")
-            res = d.run()
-            d.destroy()
-            sys.exit()
-
         self._langlist.clear()
 
         progress = GtkProgress(self.dialog_progress, self.progressbar_cache,
@@ -534,13 +681,27 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGladeApp):
             sys.exit(1)
 
         languageList = self._cache.getLanguageInformation()
+        #print "ll size: ", len(languageList)
+        #print "ll type: ", type(languageList)
         for lang in languageList:
-            inconsistent = xor(lang.langPackInstalled,lang.langSupportInstalled)
+            #print "langInfo: %s" % lang
+            inconsistent = lang.inconsistent
             #if inconsistent:
             #    print "inconsistent", lang.language
-            installed = lang.langPackInstalled or lang.langSupportInstalled
+            installed = lang.fullInstalled
             self._langlist.append([_(lang.language), lang])
         self._langlist.set_sort_column_id(LIST_LANG, gtk.SORT_ASCENDING)
+        for button in ( 
+              "checkbutton_basic_translations",
+              "checkbutton_extra_translations",
+              "checkbutton_writing_aids",
+              "checkbutton_input_methods",
+              "checkbutton_extra",
+              "checkbutton_fonts"):
+            self.block_toggle = True
+            getattr(self, button).set_active(False)
+            getattr(self, button).set_sensitive(False)
+            self.block_toggle = False
 
     def writeSystemDefaultLang(self):
         combo = self.combobox_default_lang
@@ -548,10 +709,30 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGladeApp):
         if combo.get_active() < 0:
             return
         (lang, code) = model[combo.get_active()]
-        #print "lang=%s, code=%s" % (lang, code)
-
-        # make a copy (in case we do anything bad)
+        old_code = self._localeinfo.getDefaultLanguage()
+        # no changes, nothing to do
+        if old_code == code:
+            return False
         self.setSystemDefaultLanguage(code)
+        #self.showRebootRequired()
+        self.checkReloginNotification()
+        return True
+
+    def showRebootRequired(self):
+        " show a message box that a restart is required "
+        d = gtk.MessageDialog(parent=self.window_main,
+                              flags=gtk.DIALOG_MODAL,
+                              type=gtk.MESSAGE_INFO,
+                              buttons=gtk.BUTTONS_CLOSE)
+        d.set_markup("<big><b>%s</b></big>\n\n%s" % (
+                _("Reboot required"),
+                _("To complete the changes a reboot is required so "
+                  "that the new functionality is activated.")
+                ))
+        d.set_title=("")
+        res = d.run()
+        d.destroy()
+        
 
     def updateSystemDefaultCombo(self):
         #print "updateSystemDefault()"
