@@ -6,8 +6,10 @@
 #
 
 import sys
-from PyQt4.QtCore import *
 from PyQt4.QtGui import *
+from PyQt4.QtCore import *
+from PyKDE4.kdecore import ki18n, KAboutData, KCmdLineArgs, KCmdLineOptions
+from PyKDE4.kdeui import KApplication, KIcon,  KMessageBox, KGuiItem
 
 from LanguageSelector.LanguageSelector import *
 from QtLanguageSelectorGUI import Ui_QtLanguageSelectorGUI
@@ -19,7 +21,7 @@ def utf8(str):
   return unicode(str, 'UTF-8')
 
 def _(string):
-    return unicode(i18n(string), "utf-8")
+    return utf8(i18n(string))
 
 class QtLanguageSelector(QWidget,LanguageSelectorBase):
     """ actual implementation of the qt GUI """
@@ -33,11 +35,11 @@ class QtLanguageSelector(QWidget,LanguageSelectorBase):
 
         self.parentApp = app
 
-        self.setWindowIcon(QIcon("/usr/share/icons/crystalsvg/32x32/apps/locale.png"))
-        self.ui.pushButtonSetSystemLanguage.setIcon(QIcon("/usr/share/icons/crystalsvg/22x22/actions/button_ok.png"))
-        self.ui.pushButtonOk.setIcon(QIcon("/usr/share/icons/crystalsvg/22x22/actions/button_ok.png"))
-        self.ui.pushButtonCancel.setIcon(QIcon("/usr/share/icons/crystalsvg/22x22/actions/button_cancel.png"))
-        self.ui.pushButtonCancel_2.setIcon(QIcon("/usr/share/icons/crystalsvg/22x22/actions/button_cancel.png"))
+        self.setWindowIcon(KIcon("preferences-desktop-locale"))
+        self.ui.pushButtonSetSystemLanguage.setIcon(KIcon("dialog-ok"))
+        self.ui.pushButtonOk.setIcon(KIcon("dialog-ok"))
+        self.ui.pushButtonCancel.setIcon(KIcon("dialog-cancel"))
+        self.ui.pushButtonCancel_2.setIcon(KIcon("dialog-cancel"))
 
         self.mode = mode
         self.init()
@@ -67,16 +69,39 @@ class QtLanguageSelector(QWidget,LanguageSelectorBase):
             self.openCache(apt.progress.OpProgress())
         except ExceptionPkgCacheBroken:
             s = _("Software database is broken")
-            # FIXME: mention adept here instead of synaptic, but the
-            #        change happend during string freeze
             t = _("It is impossible to install or remove any software. "
-                  "Please use the package manager \"Synaptic\" or run "
+                  "Please use the package manager \"Adept\" or run "
                   "\"sudo apt-get install -f\" in a terminal to fix "
                   "this issue at first.")
-            QMessageBox.warning(self, s, t)
+            KMessageBox.Error(self, t, s)
         self.updateLanguagesList()
         self.updateSystemDefaultListbox()
 
+        if not self.verifyPackageLists():
+            yesText = _("_Update").replace("_", "&")
+            noText = _("_Remind Me Later").replace("_", "&")
+            yes = KGuiItem(yesText, "dialog-ok")
+            no = KGuiItem(noText, "process-stop")
+            text = "<big><b>%s</b></big>\n\n%s" % (
+              _("No language information available"),
+              _("The system does not have information about the "
+                "available languages yet. Do you want to perform "
+                "a network update to get them now? "))
+            text = text.replace("\n", "<br />")
+            res = KMessageBox.questionYesNo(self, text, "", yes, no)
+            if res == KMessageBox.Yes:
+                self.setEnabled(False)
+                self.update()
+                self.openCache(apt.progress.OpProgress())
+                self.updateLanguagesList()
+                self.setEnabled(True)
+
+
+        # see if something is missing
+        if True: #options.verify_installed:
+            self.verifyInstalledLangPacks()
+
+
     def translateUI(self):
         """ translate the strings in the UI, needed because Qt designer doesn't use gettext """
         self.ui.defaultSystemLabel.setText(_("Default system language:"))
@@ -94,7 +119,7 @@ class QtLanguageSelector(QWidget,LanguageSelectorBase):
         defaultLangName = None
         defaultLangCode = self.getSystemDefaultLanguage()
         if defaultLangCode:
-            defaultLangName = self._localeinfo.translate(defaultLangCode)
+            defaultLangName = utf8(self._localeinfo.translate(defaultLangCode))
         locales = []
         for locale in self._localeinfo.generated_locales():
             name = utf8(self._localeinfo.translate(locale))
@@ -109,26 +134,86 @@ class QtLanguageSelector(QWidget,LanguageSelectorBase):
             not os.path.exists("/usr/bin/im-switch")):
             self.ui.enableInputMethods.setEnabled(False)
 
+    def verifyInstalledLangPacks(self):
+        """ called at the start to inform about possible missing
+            langpacks (e.g. gnome/kde langpack transition)
+        """
+        print "verifyInstalledLangPacks"
+        missing = []
+        
+        languageList = self._cache.getLanguageInformation()
+        #self._localeinfo.listviewStrToLangInfoMap = {}
+        for langInfo in languageList:
+            trans_package = "language-pack-%s" % langInfo.languageCode
+            # we have a langpack installed, see if we have all of it
+            # (for hoary -> breezy transition)
+            if self._cache.has_key(trans_package) and \
+               self._cache[trans_package].isInstalled:
+                #print "IsInstalled: %s " % trans_package
+                for (pkg, translation) in self._cache.pkg_translations:
+                    missing += self.missingTranslationPkgs(pkg, translation+langInfo.languageCode)
+
+        print "Missing: %s " % missing
+        if len(missing) > 0:
+            # FIXME: add "details"
+            yesText = _("_Install").replace("_", "&")
+            noText = _("_Remind Me Later").replace("_", "&")
+            yes = KGuiItem(yesText, "dialog-ok")
+            no = KGuiItem(noText, "process-stop")
+            text = "<big><b>%s</b></big>\n\n%s" % (
+                _("The language support is not installed completely"),
+                _("Some translations or writing aids available for your "
+                  "chosen languages are not installed yet. Do you want "
+                  "to install them now?"))
+            text = text.replace("\n", "<br />")
+            res = KMessageBox.questionYesNo(self, text, "", yes, no)
+            if res == KMessageBox.Yes:
+                self.setEnabled(False)
+                self.commit(missing, [])
+                self.updateLanguagesList()
+                self.setEnabled(True)
+
+    def update(self):
+        lock = thread.allocate_lock()
+        lock.acquire()
+        t = thread.start_new_thread(self.run_pkg_manager_update,(lock,))
+        while lock.locked():
+            self.parentApp.processEvents()
+            time.sleep(0.05)
+
+    def commit(self, inst, rm):
+        # unlock here to make sure that lock/unlock are always run
+        # pair-wise (and don't explode on errors)
+        if len(inst) == 0 and len(rm) == 0:
+            return
+        lock = thread.allocate_lock()
+        lock.acquire()
+        t = thread.start_new_thread(self.run_pkg_manager,(lock,inst,rm))
+        while lock.locked():
+            self.parentApp.processEvents()
+            time.sleep(0.05)
+
     def updateLanguagesList(self):
         self.ui.listViewLanguages.clear()
         # get the language names and sort them alphabetically
         languageList = self._cache.getLanguageInformation()
         self._localeinfo.listviewStrToLangInfoMap = {}
         for lang in languageList:
-            self._localeinfo.listviewStrToLangInfoMap[lang.language] = lang
+            self._localeinfo.listviewStrToLangInfoMap[utf8(lang.language)] = lang
         languages = self._localeinfo.listviewStrToLangInfoMap.keys()
         languages.sort()
 
         for langName in languages:
             lang = self._localeinfo.listviewStrToLangInfoMap[langName]
             elm = QListWidgetItem(utf8(lang.language), self.ui.listViewLanguages)
-            if lang.langPackInstalled and lang.langSupportInstalled:
+
+            if lang.fullInstalled:
                 if self.mode == "install":
                     elm.setFlags(Qt.ItemIsDropEnabled)  #not sure how to unset all flags, but this disables the item
                     elm.setToolTip(_("Already installed"))
                 else:
                     elm.setFlags(Qt.ItemIsSelectable | Qt.ItemIsEnabled)
-            elif lang.langPackInstalled or lang.langSupportInstalled:
+            elif lang.inconsistent:
                 elm.setToolTip(_("Partially Installed"))
             else:
                 if self.mode == "uninstall":
@@ -165,13 +250,19 @@ class QtLanguageSelector(QWidget,LanguageSelectorBase):
                 except KeyError:
                     print "ERROR: can not find new_locale: '%s'"%new_locale
 
+    def run_pkg_manager_update(self, lock):
+        self.returncode = 0
+        self.returncode = subprocess.call(["install-package","--update"])
+        lock.release()
+
     def run_pkg_manager(self, lock, to_inst, to_rm):
         self.returncode = 0
         if len(to_inst) > 0:
-            self.returncode = subprocess.call(["adept_batch","install"]+to_inst)
+            print str(["install-package","--install"]+to_inst)
+            self.returncode = subprocess.call(["install-package","--install"]+to_inst)
         # then remove
         if len(to_rm) > 0:
-            self.returncode = subprocess.call(["adept_batch","remove"]+to_rm)
+            self.returncode = subprocess.call(["install-package","--uninstall"]+to_rm)
         lock.release()
 
     def onSystemPushButtonOk(self):
@@ -179,9 +270,9 @@ class QtLanguageSelector(QWidget,LanguageSelectorBase):
         self.setSystemDefaultLanguage(code)
         scimOn = self.updateInputMethods()
         if scimOn:
-            QMessageBox.information(self, _("Language Set"), _("Default system Language now set to %s.  Complex character input will be enabled when you next log in.") % lang)
+            KMessageBox.information(self, _("Default system Language now set to %s.  Complex character input will be enabled when you next log in." % lang), _("Language Set"))
         else:
-            QMessageBox.information(self, _("Language Set"), _("Default system Language now set to %s.") % lang)
+            KMessageBox.information(self, _("Default system Language now set to %s.") % lang, _("Language Set"))
         self.close()
 
     def __input_method_config_changed(self):
@@ -210,7 +301,7 @@ class QtLanguageSelector(QWidget,LanguageSelectorBase):
         # if something has changed - act!
         if self.__input_method_config_changed():
             if self.ui.enableInputMethods.checkState() == Qt.Checked:
-                os.system("im-switch -z %s -s scim" % code)
+                os.system("im-switch -z %s -s scim-bridge" % code)
                 return True
             else:
                 os.system("im-switch -z %s -a" % code)
@@ -233,7 +324,7 @@ class QtLanguageSelector(QWidget,LanguageSelectorBase):
         items = self.ui.listViewLanguages.selectedItems()
         if len(items) == 1:
             elm = items[0]
-            li = self._localeinfo.listviewStrToLangInfoMap["%s"%elm.text().toUtf8()]
+            li = self._localeinfo.listviewStrToLangInfoMap[unicode(elm.text())]
             if self.mode == "uninstall":
                 self._cache.tryRemoveLanguage(li.languageCode)
             else:
@@ -262,22 +353,63 @@ class QtLanguageSelector(QWidget,LanguageSelectorBase):
 
         #self.run_pkg_manager(to_inst, to_rm)
         if self.returncode == 0 and self.mode == "install":
-            QMessageBox.information( self, _("Language Installed"), _("Translations and support have now been installed for %s.  Select them from the Add Language button." % str(items[0].text())) )
+            KMessageBox.information( self, _("Translations and support have now been installed for %s.  Select them from the Add Language button.") % unicode(items[0].text()), _("Language Installed") )
         elif self.returncode == 0 and self.mode == "uninstall":
-            QMessageBox.information( self, _("Language Uninstalled"), _("Translations and support have now been uninstalled for %s." % str(items[0].text())) )
+            KMessageBox.information( self, _("Translations and support have now been uninstalled for %s.") % unicode(items[0].text()), _("Language Uninstalled") )
         else:
-            QMessageBox.warning(self, _("Language Not Set"), _("Failed to set system language."))
+            KMessageBox.sorry(self, _("Failed to set system language."), _("Language Not Set"))
         self.close()
 
 if __name__ == "__main__":
 
-    app = QApplication(sys.argv)
+    appName	= "language-selector"
+    catalog	= ""
+    programName = ki18n ("Language Selector")
+    version	= "0.3.4"
+    description	= ki18n ("Language Selector")
+    license	= KAboutData.License_GPL
+    copyright	= ki18n ("(c) 2008 Canonical Ltd")
+    text	= ki18n ("none")
+    homePage	= "https://launchpad.net/language-selector"
+    bugEmail	= ""
     
-    lc = QtLanguageSelector(app, "/usr/share/language-selector/", sys.argv[2])
-    app.setMainWidget(lc)
+    aboutData	= KAboutData (appName, catalog, programName, version, description, license, copyright, text, homePage, bugEmail)
+    
+    aboutData.addAuthor(ki18n("Rob Bean"), ki18n("PyQt4 to PyKDE4 port"))
+
+    options = KCmdLineOptions()
+    options.add("!mode ", ki18n("REQUIRED: install, uninstall or select must follow"),  "select")
+    options.add("+[install]", ki18n("install a language"))
+    options.add("+[uninstall]", ki18n("uninstall a language"))
+    options.add("+[select]", ki18n("select a language"))
+
+    KCmdLineArgs.init (sys.argv, aboutData)
+    KCmdLineArgs.addCmdLineOptions(options)
+
+    gettext.bindtextdomain("language-selector", "/usr/share/locale")
+    gettext.textdomain("language-selector")
+
+    app = KApplication()
+    
+    args = KCmdLineArgs.parsedArgs()
+    
+    if args.isSet("mode"):
+        whattodo = args.getOption("mode")
+        if whattodo in ["install", "uninstall", "select"]:
+            pass
+        else:
+            print whattodo, "is not a valid argument"
+            args.usage()
+    else:
+        print "Please review the usage."
+        args.usage()
+        
+    if os.getuid() != 0:
+        KMessageBox.sorry(None, _("Please run this software with administrative rights."),  _("Not Root User"))
+        sys.exit(1)
+
+    lc = QtLanguageSelector(app, "/usr/share/language-selector/", whattodo)
     
     lc.show()
 
-
-    app.exec_loop()
-    
+    app.exec_()
